<?php
/**
* @file
* @brief    Liturgical (Christian) year module.
* @package  mod_liturgicalyear
* @author   Levente Hunyadi
* @version  1.0.5
* @remarks  Copyright (C) 2009-2011 Levente Hunyadi
* @remarks  Licensed under GNU/GPLv3, see http://www.gnu.org/licenses/gpl-3.0.html
* @see      http://hunyadi.info.hu/projects/liturgicalyear
*/

// no direct access
defined('_JEXEC') or die('Restricted access');

jimport('joomla.utilities.date');  // JDate

/**
* Julian day count corresponding to Gregorian date part of local time.
* The function uses the time zone set in the Joomla global configuration file.
* @return Julian day count according to local time.
*/
function jd_get_local_date() {
	// get a date object based on the correct timezone
	$config = JFactory::getConfig();
	$timezone = new DateTimeZone($config->get('offset'));  // get timezone from configuration
	$jdate = JFactory::getDate('now', 'UTC');
	$jdate->setTimezone($timezone);

	$year = (int)$jdate->format('Y', true);  // use local time
	$month = (int)$jdate->format('m', true);
	$day = (int)$jdate->format('d', true);
	return gregoriantojd($month, $day, $year);
}

/**
* The date no later than a specified date that falls on the given day of week.
* @param jd The date in Julian day count.
* @param weekday The day of week, 0 to 6 inclusive, with 0 = Sunday and 6 = Saturday.
* @return The date sought in Julian day count.
*/
function jd_get_weekday_before($jd, $weekday) {
	$day_of_week = jddayofweek($jd);
	if ($weekday > $day_of_week) {
		return $jd + $weekday - $day_of_week - 7;
	} elseif ($day_of_week > $weekday) {
		return $jd + $weekday - $day_of_week;
	} else {
		return $jd;
	}
}

/**
* The date no earlier than a specified date that falls on the given day of week.
* @param jd The date in Julian day count.
* @param weekday The day of week, 0 to 6 inclusive, with 0 = Sunday and 6 = Saturday.
* @return The date sought in Julian day count.
*/
function jd_get_weekday_after($jd, $weekday) {
	$day_of_week = jddayofweek($jd);
	if ($weekday > $day_of_week) {
		return $jd + $weekday - $day_of_week;
	} elseif ($day_of_week > $weekday) {
		return $jd + $weekday - $day_of_week + 7;
	} else {
		return $jd;
	}
}

/**
* The last day in a month that falls on the given day of week.
* @param weekday The day of week, 0 to 6 inclusive, with 0 = Sunday and 6 = Saturday.
* @return A date in Julian day count.
*/
function jd_get_last_weekday($year, $month, $weekday) {
	$days = cal_days_in_month(CAL_GREGORIAN, $month, $year);  /* number of days in month */
	$jd = gregoriantojd($month, $days, $year);  /* last day in month */
	return jd_get_weekday_before($jd, $weekday);
}

/**
* The first day in a month that falls on the given day of week.
* @param weekday The day of week, 0 to 6 inclusive, with 0 = Sunday and 6 = Saturday.
* @return A date in Julian day count.
*/
function jd_get_first_weekday($year, $month, $weekday) {
	$jd = gregoriantojd($month, 1, $year);  /* first day in month */
	return jd_get_weekday_after($jd, $weekday);
}

/**
* Modified Oudin's algorithm for calculating the date of Easter in the Gregorian calendar.
* @param year The year for which to return the date of Easter.
* @return A date in Julian day count.
*/
function jd_get_easter_date($year) {
	$century = idiv($year,100);
	$G = $year % 19;
	$K = idiv(($century - 17),25);
	$I = ($century - idiv($century,4) - idiv(($century - $K),3) + 19*$G + 15) % 30;
	$I = $I - idiv($I,28)*(1 - idiv($I,28)*idiv(29,($I + 1))*idiv((21 - $G),11));
	$J = ($year + idiv($year,4) + $I + 2 - $century + idiv($century,4)) % 7;
	$L = $I - $J;
	$month = 3 + idiv(($L + 40),44);
	$day = $L + 28 - 31*idiv($month,4);
	return gregoriantojd($month, $day, $year);
}

/**
* Integer division.
*/
function idiv($x, $y) {
	$x = (int) $x;
	$y = (int) $y;
    return ($x - ($x % $y)) / $y;
}

class modLiturgicalYearHelper {
	/**
	* List of denominations.
	* Each solemnity may be assigned to one or more denominations.
	*/
	public static function get_denominations() {
		return array(
			'commonly_observed',  // commonly observed solemnities
			'catholic',
			'calvinist',
			'lutheran');
	}

	/**
	* List of events observed by a given denomination.
	* @param denomination A denomination key returned by get_denominations.
	*/
	public static function get_event_observance($denomination) {
		$events = array(
			'reformation_day'=>array('calvinist','lutheran'),
			'all_saints_day'=>array('catholic'),
			'all_souls_day'=>array('catholic','lutheran'),
			'eternity_sunday'=>array('lutheran'),
			'advent_1st'=>array('commonly_observed'),
			'advent_2nd'=>array('commonly_observed'),
			'advent_3rd'=>array('commonly_observed'),
			'advent_4th'=>array('commonly_observed'),
			'christmas_1st'=>array('commonly_observed'),
			'christmas_2nd'=>array('commonly_observed'),
			'epiphany'=>array('commonly_observed'),
			'ash_wednesday'=>array('catholic'),
			'lent_sunday'=>array('commonly_observed'),
			'palm_sunday'=>array('commonly_observed'),
			'good_thursday'=>array('commonly_observed'),
			'good_friday'=>array('commonly_observed'),
			'holy_saturday'=>array('catholic'),
			'easter'=>array('commonly_observed'),
			'ascension'=>array('commonly_observed'),
			'pentecost'=>array('commonly_observed'),
			'whit_monday'=>array('commonly_observed'),
			'trinity_sunday'=>array('catholic'),
			'new_bread'=>array('calvinist','lutheran'),
			'new_wine'=>array('calvinist','lutheran')
		);
		$observed_events = array();
		foreach ($events as $event_key => $event_observance) {
			if (in_array($denomination, $event_observance, true)) {
				$observed_events[] = $event_key;
			}
		}
		return $observed_events;
	}

	public static function get_event_dates($year) {
		$jd_christmaseve = gregoriantojd(12, 24, $year);
		$jd_easter = jd_get_easter_date($year);  /* Easter date in Julian calendar */
		return array(
			'reformation_day'=>gregoriantojd(10, 31, $year),  /* a Reformáció napja */
			'all_saints_day'=>gregoriantojd(11, 1, $year),  /* Mindenszentek ünnepe */
			'all_souls_day'=>gregoriantojd(11, 2, $year),  /* halottak napja */
			'eternity_sunday'=>jd_get_weekday_before($jd_christmaseve-28, 0),
			'advent_1st'=>jd_get_weekday_before($jd_christmaseve-21, 0),  /* advent első vasárnapja */
			'advent_2nd'=>jd_get_weekday_before($jd_christmaseve-14, 0),  /* advent második vasárnapja */
			'advent_3rd'=>jd_get_weekday_before($jd_christmaseve-7, 0),  /* advent harmadik vasárnapja */
			'advent_4th'=>jd_get_weekday_before($jd_christmaseve, 0),  /* advent negyedik vasárnapja */
			'christmas_1st'=>$jd_christmaseve+1,  /* karácsony */
			'christmas_2nd'=>$jd_christmaseve+2,  /* karácsony */
			'epiphany'=>gregoriantojd(1, 6, $year),  /* vízkereszt */
			'ash_wednesday'=>$jd_easter-46,  /* hamvazószerda */
			'lent_sunday'=>$jd_easter-42,  /* nagyböjt első vasárnapja */
			'palm_sunday'=>$jd_easter-7,  /* virágvasárnap */
			'good_thursday'=>$jd_easter-3,  /* nagycsütörtök */
			'good_friday'=>$jd_easter-2,  /* nagypéntek */
			'holy_saturday'=>$jd_easter-1,  /* nagyszombat */
			'easter'=>$jd_easter,  /* húsvét */
			'ascension'=>$jd_easter+39,  /* mennybemenetel, áldozócsütörtök */
			'pentecost'=>$jd_easter+49,  /* pünkösd */
			'whit_monday'=>$jd_easter+50,  /* pünkösd második napja */
			'trinity_sunday'=>$jd_easter+56,  /* a Szentháromság ünnepe */
			'new_bread'=>jd_get_weekday_before(gregoriantojd(8, 31, $year), 0),  /* az új kenyér ünnepe, augusztus utolsó vasárnapja */
			'new_wine'=>jd_get_weekday_before(gregoriantojd(10, 31, $year), 0)  /* az új bor ünnepe, október utolsó vasárnapja */
		);
	}
}